// SPDX-License-Identifier: GPL-2.0
/*
* Copyright (C) 2020 Red Hat. All rights reserved.
*
* This file is released under the GPL.
*/

#include "nvfs.h"

int nvfs_fileattr_get(struct dentry *dentry, struct fileattr *fa)
{
	struct inode *inode = d_inode(dentry);
	struct nvfs_memory_inode *nmi = i_to_nmi(inode);
	fileattr_fill_flags(fa, nmi->i_flag & (FS_FL_USER_VISIBLE & NVFS_FLAGS));
	return 0;
}

int nvfs_fileattr_set(struct user_namespace *mnt_userns, struct dentry *dentry, struct fileattr *fa)
{
	struct inode *inode = d_inode(dentry);
	struct nvfs_memory_inode *nmi = i_to_nmi(inode);
	struct nvfs_superblock *nvs = inode->i_sb->s_fs_info;
	unsigned flags, oldflags;

	if (fileattr_has_fsx(fa))
		return -EOPNOTSUPP;

	if (IS_NOQUOTA(inode))
		return -EPERM;

	flags = nvfs_mask_flags(inode->i_mode, fa->flags);
	oldflags = nmi->i_flag;
	flags = flags & (FS_FL_USER_MODIFIABLE & NVFS_FLAGS);
	flags |= oldflags & ~(FS_FL_USER_MODIFIABLE | NVFS_FLAGS);
	nmi->i_flag = flags;
	nv_store(nvs, nmi->nvi->i_flag, cpu_to_le32(flags));

	nvfs_set_inode_flags(inode);
	nvfs_update_time(inode, NULL, S_CTIME);

	return 0;
}

long nvfs_ioctl(struct file *filp, unsigned cmd, unsigned long arg)
{
	return -ENOTTY;
}

#ifdef CONFIG_COMPAT
long nvfs_compat_ioctl(struct file *file, unsigned cmd, unsigned long arg)
{
	return -ENOIOCTLCMD;
}
#endif
