// SPDX-License-Identifier: GPL-2.0
/*
* Copyright (C) 2020 Red Hat. All rights reserved.
*
* This file is released under the GPL.
*/

#include "nvfs.h"

void nvfs_memzero_flushcache(struct nvfs_superblock *nvs, char *ptr, size_t len)
{
#if 0
	/* this doesn't seem to have any effect */
	if (len >= 128) {
		size_t len1 = 64 - ((unsigned long)ptr & 63);
		memset(ptr, 0, len1);
		clwb(ptr);
		ptr += len1;
		len -= len1;
		do {
			((__u64 *)ptr)[0] = 0;
			((__u64 *)ptr)[1] = 0;
			((__u64 *)ptr)[2] = 0;
			((__u64 *)ptr)[3] = 0;
			((__u64 *)ptr)[4] = 0;
			((__u64 *)ptr)[5] = 0;
			((__u64 *)ptr)[6] = 0;
			((__u64 *)ptr)[7] = 0;
			clwb(ptr);
			ptr += 64;
			len -= 64;
		} while (len >= 64);
	}
#endif
	if (len) {
		memset(ptr, 0, len);
		nv_flush(nvs, ptr, len);
	}
}

uint64_t nvfs_get_random(uint64_t limit)
{
	if (limit < 0x1000000) {
		return prandom_u32() % (uint32_t)limit;
	} else {
		uint64_t result;
		uint64_t v = prandom_u32() | ((uint64_t)prandom_u32() << 32);
		div64_u64_rem(v, limit, &result);
		return result;
	}
}

unsigned long nvfs_find_bit_in_bmp(uint8_t *bitmap, unsigned long start, unsigned long end)
{
	unsigned long o;
	if (!test_bit_le(start, bitmap))
		return start;

	cond_resched();
	o = find_next_zero_bit_le(bitmap, end, start);
	if (o < end)
		return o;
	cond_resched();
	o = find_next_zero_bit_le(bitmap, start, 0);
	if (o < start)
		return o;
	cond_resched();

	return ~0;
}
