// SPDX-License-Identifier: GPL-2.0
/*
* Copyright (C) 2020 Red Hat. All rights reserved.
*
* This file is released under the GPL.
*/

#include "fsck.h"

void *fsck_malloc(size_t s)
{
	void *p;
	p = malloc(s);
	if (!p) {
		fsck_log("malloc(%lu) failed: %s", (unsigned long)s, strerror(errno));
		exit(8);
	}
	return p;
}

void *fsck_realloc(void *p, size_t s)
{
	p = realloc(p, s);
	if (!p) {
		fsck_log("realloc(%lu) failed: %s", (unsigned long)s, strerror(errno));
		exit(8);
	}
	return p;
}

void fsck_free(void *p)
{
	free(p);
}

void init_array(uint64_t **a, size_t *len)
{
	*a = fsck_malloc(0);
	*len = 0;
}

void append_array(uint64_t **a, size_t *len, uint64_t val)
{
	if (!(*len & (*len - 1))) {
		size_t newlen = *len * 2;
		if (!newlen)
			newlen = 1;
		*a = fsck_realloc(*a, newlen * sizeof(uint64_t));
	}
	(*a)[(*len)++] = val;
}

void *fsck_mmap(size_t s)
{
	void *p;
	p = mmap(NULL, s, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
	if (p == MAP_FAILED) {
		fsck_log("mmap(%lu) failed: %s", (unsigned long)s, strerror(errno));
		exit(8);
	}
	return p;
}

void fsck_munmap(void *p, size_t s)
{
	int r;
	r = munmap(p, s);
	if (r)
		fsck_internal_error("munmap failed: %s", strerror(errno)), exit(8);
}
