// SPDX-License-Identifier: GPL-2.0
/*
* Copyright (C) 2020 Red Hat. All rights reserved.
*
* This file is released under the GPL.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <mntent.h>

#include "mounted.h"

static struct stat st;

static int stateq(const struct stat *st1, const struct stat *st2)
{
	if (st1->st_dev == st2->st_dev && st1->st_ino == st2->st_ino)
		return 1;
	if (S_ISBLK(st1->st_mode) && S_ISBLK(st2->st_mode) && st1->st_rdev == st2->st_rdev)
		return 1;
	if (S_ISCHR(st1->st_mode) && S_ISCHR(st2->st_mode) && st1->st_rdev == st2->st_rdev)
		return 1;
	return 0;
}

static int is_mounted_in(FILE *mnt, const char *dev)
{
	int ret = 0;
	struct mntent *mntent;

	while ((mntent = getmntent(mnt))) {
		int val;
		struct stat stm;

		//fprintf(stderr, "fsname '%s', mnt_dir '%s'\n", mntent->mnt_fsname, mntent->mnt_dir);

		if (mntent->mnt_fsname[0] != '/')
			continue;

		if (hasmntopt(mntent, MNTOPT_RO))
			val = 1;
		else
			val = 2;

		if (!strcmp(mntent->mnt_fsname, dev)) {
			ret = val;
			break;
		}

		if (!stat(mntent->mnt_fsname, &stm)) {
			if (stateq(&st, &stm)) {
				ret = val;
				break;
			}
		}

		if (S_ISBLK(st.st_mode) && !strcmp(mntent->mnt_fsname, "/dev/root") && !stat(mntent->mnt_dir, &stm)) {
			if (st.st_rdev == stm.st_dev) {
				ret = val;
				break;
			}
		}
	}

	endmntent(mnt);
	return ret;
}

int is_mounted(const char *dev, int h)
{
	FILE *mnt;
	if (fstat(h, &st)) {
		perror("fstat");
		exit(1);
	}
	//if ((mnt = setmntent("/proc/mounts", "r"))) return is_mounted_in(mnt, dev);
	if ((mnt = setmntent(_PATH_MOUNTED, "r"))) return is_mounted_in(mnt, dev);
	return 0;
}
